"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .versionheader import VersionHeader
from gusto_embedded.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from gusto_embedded.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    RequestMetadata,
)
import pydantic
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class PutV1EmployeesEmployeeIDFederalTaxesRequestBodyTypedDict(TypedDict):
    version: str
    r"""The current version of the object. See the [versioning guide](https://docs.gusto.com/embedded-payroll/docs/versioning#object-layer) for information on how to use this field."""
    filing_status: NotRequired[str]
    extra_withholding: NotRequired[Nullable[str]]
    two_jobs: NotRequired[bool]
    dependents_amount: NotRequired[str]
    other_income: NotRequired[str]
    deductions: NotRequired[str]
    w4_data_type: NotRequired[str]


class PutV1EmployeesEmployeeIDFederalTaxesRequestBody(BaseModel):
    version: str
    r"""The current version of the object. See the [versioning guide](https://docs.gusto.com/embedded-payroll/docs/versioning#object-layer) for information on how to use this field."""

    filing_status: Optional[str] = None

    extra_withholding: OptionalNullable[str] = UNSET

    two_jobs: Optional[bool] = None

    dependents_amount: Optional[str] = None

    other_income: Optional[str] = None

    deductions: Optional[str] = None

    w4_data_type: Optional[str] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "filing_status",
            "extra_withholding",
            "two_jobs",
            "dependents_amount",
            "other_income",
            "deductions",
            "w4_data_type",
        ]
        nullable_fields = ["extra_withholding"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class PutV1EmployeesEmployeeIDFederalTaxesRequestTypedDict(TypedDict):
    employee_uuid: str
    r"""The UUID of the employee"""
    request_body: PutV1EmployeesEmployeeIDFederalTaxesRequestBodyTypedDict
    x_gusto_api_version: NotRequired[VersionHeader]
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""


class PutV1EmployeesEmployeeIDFederalTaxesRequest(BaseModel):
    employee_uuid: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The UUID of the employee"""

    request_body: Annotated[
        PutV1EmployeesEmployeeIDFederalTaxesRequestBody,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]

    x_gusto_api_version: Annotated[
        Optional[VersionHeader],
        pydantic.Field(alias="X-Gusto-API-Version"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .sort_order import SortOrder
from .versionheader import VersionHeader
from enum import Enum
from gusto_embedded.types import BaseModel
from gusto_embedded.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    QueryParamMetadata,
)
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class ProcessingStatuses(str, Enum):
    UNPROCESSED = "unprocessed"
    PROCESSED = "processed"


class PayrollTypes(str, Enum):
    REGULAR = "regular"
    OFF_CYCLE = "off_cycle"
    EXTERNAL = "external"


class GetV1CompaniesCompanyIDPayrollsQueryParamInclude(str, Enum):
    TOTALS = "totals"
    PAYROLL_STATUS_META = "payroll_status_meta"
    RISK_BLOCKERS = "risk_blockers"


class GetV1CompaniesCompanyIDPayrollsRequestTypedDict(TypedDict):
    company_id: str
    r"""The UUID of the company"""
    processing_statuses: NotRequired[List[ProcessingStatuses]]
    r"""Whether to include processed and/or unprocessed payrolls in the response, defaults to processed, for multiple attributes comma separate the values, i.e. `?processing_statuses=processed,unprocessed`"""
    payroll_types: NotRequired[List[PayrollTypes]]
    r"""Whether to include regular and/or off_cycle payrolls in the response, defaults to regular, for multiple attributes comma separate the values, i.e. `?payroll_types=regular,off_cycle`"""
    include: NotRequired[List[GetV1CompaniesCompanyIDPayrollsQueryParamInclude]]
    r"""Include the requested attribute in the response. The risk_blockers option will include submission_blockers and credit_blockers if applicable. In v2023-04-01 totals are no longer included by default. For multiple attributes comma separate the values, i.e. `?include=totals,payroll_status_meta`"""
    start_date: NotRequired[str]
    r"""Return payrolls whose pay period is after the start date"""
    end_date: NotRequired[str]
    r"""Return payrolls whose pay period is before the end date. If left empty, defaults to today's date."""
    sort_order: NotRequired[SortOrder]
    r"""A string indicating whether to sort resulting events in ascending (asc) or descending (desc) chronological order. Events are sorted by their `timestamp`. Defaults to asc if left empty."""
    page: NotRequired[int]
    r"""The page that is requested. When unspecified, will load all objects unless endpoint forces pagination."""
    per: NotRequired[int]
    r"""Number of objects per page. For majority of endpoints will default to 25"""
    x_gusto_api_version: NotRequired[VersionHeader]
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""


class GetV1CompaniesCompanyIDPayrollsRequest(BaseModel):
    company_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The UUID of the company"""

    processing_statuses: Annotated[
        Optional[List[ProcessingStatuses]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Whether to include processed and/or unprocessed payrolls in the response, defaults to processed, for multiple attributes comma separate the values, i.e. `?processing_statuses=processed,unprocessed`"""

    payroll_types: Annotated[
        Optional[List[PayrollTypes]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Whether to include regular and/or off_cycle payrolls in the response, defaults to regular, for multiple attributes comma separate the values, i.e. `?payroll_types=regular,off_cycle`"""

    include: Annotated[
        Optional[List[GetV1CompaniesCompanyIDPayrollsQueryParamInclude]],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=False)),
    ] = None
    r"""Include the requested attribute in the response. The risk_blockers option will include submission_blockers and credit_blockers if applicable. In v2023-04-01 totals are no longer included by default. For multiple attributes comma separate the values, i.e. `?include=totals,payroll_status_meta`"""

    start_date: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Return payrolls whose pay period is after the start date"""

    end_date: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Return payrolls whose pay period is before the end date. If left empty, defaults to today's date."""

    sort_order: Annotated[
        Optional[SortOrder],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""A string indicating whether to sort resulting events in ascending (asc) or descending (desc) chronological order. Events are sorted by their `timestamp`. Defaults to asc if left empty."""

    page: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""The page that is requested. When unspecified, will load all objects unless endpoint forces pagination."""

    per: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Number of objects per page. For majority of endpoints will default to 25"""

    x_gusto_api_version: Annotated[
        Optional[VersionHeader],
        pydantic.Field(alias="X-Gusto-API-Version"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .versionheader import VersionHeader
from enum import Enum
from gusto_embedded.types import BaseModel
from gusto_embedded.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    RequestMetadata,
)
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class OffCycleReason(str, Enum):
    r"""An off cycle payroll reason. Select one from the following list."""

    BONUS = "Bonus"
    CORRECTION = "Correction"
    DISMISSED_EMPLOYEE = "Dismissed employee"
    TRANSITION_FROM_OLD_PAY_SCHEDULE = "Transition from old pay schedule"


class WithholdingPayPeriod(str, Enum):
    r"""The payment schedule tax rate the payroll is based on."""

    EVERY_WEEK = "Every week"
    EVERY_OTHER_WEEK = "Every other week"
    TWICE_PER_MONTH = "Twice per month"
    MONTHLY = "Monthly"
    QUARTERLY = "Quarterly"
    SEMIANNUALLY = "Semiannually"
    ANNUALLY = "Annually"


class PostV1CompaniesCompanyIDPayrollsRequestBodyTypedDict(TypedDict):
    off_cycle: bool
    r"""Whether it is an off cycle payroll."""
    off_cycle_reason: OffCycleReason
    r"""An off cycle payroll reason. Select one from the following list."""
    start_date: str
    r"""Pay period start date."""
    end_date: str
    r"""Pay period end date."""
    pay_schedule_uuid: NotRequired[str]
    r"""A pay schedule is required for transition from old pay schedule payroll to identify the matching transition pay period."""
    employee_uuids: NotRequired[List[str]]
    r"""A list of employee uuids to include on the payroll."""
    check_date: NotRequired[str]
    r"""Payment date."""
    withholding_pay_period: NotRequired[WithholdingPayPeriod]
    r"""The payment schedule tax rate the payroll is based on."""
    skip_regular_deductions: NotRequired[bool]
    r"""Block regular deductions and contributions for this payroll."""
    fixed_withholding_rate: NotRequired[bool]
    r"""Enable taxes to be withheld at the IRS's required rate of 22% for federal income taxes. State income taxes will be taxed at the state's supplemental tax rate. Otherwise, we'll sum the entirety of the employee's wages and withhold taxes on the entire amount at the rate for regular wages."""


class PostV1CompaniesCompanyIDPayrollsRequestBody(BaseModel):
    off_cycle: bool
    r"""Whether it is an off cycle payroll."""

    off_cycle_reason: OffCycleReason
    r"""An off cycle payroll reason. Select one from the following list."""

    start_date: str
    r"""Pay period start date."""

    end_date: str
    r"""Pay period end date."""

    pay_schedule_uuid: Optional[str] = None
    r"""A pay schedule is required for transition from old pay schedule payroll to identify the matching transition pay period."""

    employee_uuids: Optional[List[str]] = None
    r"""A list of employee uuids to include on the payroll."""

    check_date: Optional[str] = None
    r"""Payment date."""

    withholding_pay_period: Optional[WithholdingPayPeriod] = None
    r"""The payment schedule tax rate the payroll is based on."""

    skip_regular_deductions: Optional[bool] = None
    r"""Block regular deductions and contributions for this payroll."""

    fixed_withholding_rate: Optional[bool] = None
    r"""Enable taxes to be withheld at the IRS's required rate of 22% for federal income taxes. State income taxes will be taxed at the state's supplemental tax rate. Otherwise, we'll sum the entirety of the employee's wages and withhold taxes on the entire amount at the rate for regular wages."""


class PostV1CompaniesCompanyIDPayrollsRequestTypedDict(TypedDict):
    company_id: str
    r"""The UUID of the company"""
    request_body: PostV1CompaniesCompanyIDPayrollsRequestBodyTypedDict
    x_gusto_api_version: NotRequired[VersionHeader]
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""


class PostV1CompaniesCompanyIDPayrollsRequest(BaseModel):
    company_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The UUID of the company"""

    request_body: Annotated[
        PostV1CompaniesCompanyIDPayrollsRequestBody,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]

    x_gusto_api_version: Annotated[
        Optional[VersionHeader],
        pydantic.Field(alias="X-Gusto-API-Version"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .payroll_blockers_error import PayrollBlockersErrorData
from .unprocessable_entity_error_object import UnprocessableEntityErrorObjectData
from .versionheader import VersionHeader
from gusto_embedded import utils
from gusto_embedded.types import BaseModel
from gusto_embedded.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    RequestMetadata,
)
import pydantic
from typing import List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


class SubmissionBlockersTypedDict(TypedDict):
    blocker_type: NotRequired[str]
    r"""The type of submission_blocker that is blocking the payment."""
    selected_option: NotRequired[str]
    r"""The selected option to unblock the payment's submission_blocker."""


class SubmissionBlockers(BaseModel):
    blocker_type: Optional[str] = None
    r"""The type of submission_blocker that is blocking the payment."""

    selected_option: Optional[str] = None
    r"""The selected option to unblock the payment's submission_blocker."""


class PutV1CompaniesCompanyIDPayrollsPayrollIDSubmitRequestBodyTypedDict(TypedDict):
    submission_blockers: NotRequired[List[SubmissionBlockersTypedDict]]
    r"""An array of submission_blockers, each with a selected unblock option."""


class PutV1CompaniesCompanyIDPayrollsPayrollIDSubmitRequestBody(BaseModel):
    submission_blockers: Optional[List[SubmissionBlockers]] = None
    r"""An array of submission_blockers, each with a selected unblock option."""


class PutV1CompaniesCompanyIDPayrollsPayrollIDSubmitRequestTypedDict(TypedDict):
    company_id: str
    r"""The UUID of the company"""
    payroll_id: str
    r"""The UUID of the payroll"""
    request_body: PutV1CompaniesCompanyIDPayrollsPayrollIDSubmitRequestBodyTypedDict
    x_gusto_api_version: NotRequired[VersionHeader]
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""


class PutV1CompaniesCompanyIDPayrollsPayrollIDSubmitRequest(BaseModel):
    company_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The UUID of the company"""

    payroll_id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The UUID of the payroll"""

    request_body: Annotated[
        PutV1CompaniesCompanyIDPayrollsPayrollIDSubmitRequestBody,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]

    x_gusto_api_version: Annotated[
        Optional[VersionHeader],
        pydantic.Field(alias="X-Gusto-API-Version"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""


PutV1CompaniesCompanyIDPayrollsPayrollIDSubmitResponseBodyUnion = TypeAliasType(
    "PutV1CompaniesCompanyIDPayrollsPayrollIDSubmitResponseBodyUnion",
    Union[UnprocessableEntityErrorObjectData, PayrollBlockersErrorData],
)
r"""Unprocessable Entity"""


class PutV1CompaniesCompanyIDPayrollsPayrollIDSubmitResponseBody(Exception):
    r"""Unprocessable Entity"""

    data: PutV1CompaniesCompanyIDPayrollsPayrollIDSubmitResponseBodyUnion

    def __init__(
        self, data: PutV1CompaniesCompanyIDPayrollsPayrollIDSubmitResponseBodyUnion
    ):
        self.data = data

    def __str__(self) -> str:
        return utils.marshal_json(
            self.data, PutV1CompaniesCompanyIDPayrollsPayrollIDSubmitResponseBodyUnion
        )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gusto_embedded.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class PayrollFixedCompensationTypesTypeTypedDict(TypedDict):
    name: NotRequired[str]
    r"""The name of an available type of fixed compensation."""


class PayrollFixedCompensationTypesType(BaseModel):
    name: Optional[str] = None
    r"""The name of an available type of fixed compensation."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gusto_embedded.types import BaseModel
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class SchemaTypedDict(TypedDict):
    holiday_key: NotRequired[str]
    r"""the holiday's identifier"""
    holiday_name: NotRequired[str]
    r"""the holiday's official name"""
    start_date: NotRequired[str]
    r"""the holiday's start date (YYYY-MM-DD)"""
    end_date: NotRequired[str]
    r"""the holiday's end date (YYYY-MM-DD)"""


class Schema(BaseModel):
    holiday_key: Optional[str] = None
    r"""the holiday's identifier"""

    holiday_name: Optional[str] = None
    r"""the holiday's official name"""

    start_date: Optional[str] = None
    r"""the holiday's start date (YYYY-MM-DD)"""

    end_date: Optional[str] = None
    r"""the holiday's end date (YYYY-MM-DD)"""


class PaidHolidaysTypedDict(TypedDict):
    r"""Representation of a company's paid holidays as descibed by their Holiday Pay Policy"""

    schema_: NotRequired[SchemaTypedDict]


class PaidHolidays(BaseModel):
    r"""Representation of a company's paid holidays as descibed by their Holiday Pay Policy"""

    schema_: Annotated[Optional[Schema], pydantic.Field(alias="schema")] = None



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .versionheader import VersionHeader
from enum import Enum
from gusto_embedded.types import BaseModel
from gusto_embedded.utils import (
    FieldMetadata,
    HeaderMetadata,
    RequestMetadata,
    SecurityMetadata,
)
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class PostV1WebhookSubscriptionSecurityTypedDict(TypedDict):
    system_access_auth: str


class PostV1WebhookSubscriptionSecurity(BaseModel):
    system_access_auth: Annotated[
        str,
        FieldMetadata(
            security=SecurityMetadata(
                scheme=True,
                scheme_type="http",
                sub_type="bearer",
                field_name="Authorization",
            )
        ),
    ]


class PostV1WebhookSubscriptionSubscriptionTypes(str, Enum):
    BANK_ACCOUNT = "BankAccount"
    COMPANY = "Company"
    COMPANY_BENEFIT = "CompanyBenefit"
    CONTRACTOR = "Contractor"
    CONTRACTOR_PAYMENT = "ContractorPayment"
    EMPLOYEE = "Employee"
    EMPLOYEE_BENEFIT = "EmployeeBenefit"
    EMPLOYEE_JOB_COMPENSATION = "EmployeeJobCompensation"
    EXTERNAL_PAYROLL = "ExternalPayroll"
    FORM = "Form"
    LOCATION = "Location"
    NOTIFICATION = "Notification"
    PAYROLL = "Payroll"
    PAY_SCHEDULE = "PaySchedule"
    SIGNATORY = "Signatory"


class PostV1WebhookSubscriptionRequestBodyTypedDict(TypedDict):
    url: str
    subscription_types: List[PostV1WebhookSubscriptionSubscriptionTypes]


class PostV1WebhookSubscriptionRequestBody(BaseModel):
    url: str

    subscription_types: List[PostV1WebhookSubscriptionSubscriptionTypes]


class PostV1WebhookSubscriptionRequestTypedDict(TypedDict):
    request_body: PostV1WebhookSubscriptionRequestBodyTypedDict
    x_gusto_api_version: NotRequired[VersionHeader]
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""


class PostV1WebhookSubscriptionRequest(BaseModel):
    request_body: Annotated[
        PostV1WebhookSubscriptionRequestBody,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]

    x_gusto_api_version: Annotated[
        Optional[VersionHeader],
        pydantic.Field(alias="X-Gusto-API-Version"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .payroll_blockers_error import PayrollBlockersErrorData
from .unprocessable_entity_error_object import UnprocessableEntityErrorObjectData
from .versionheader import VersionHeader
from enum import Enum
from gusto_embedded import utils
from gusto_embedded.types import BaseModel
from gusto_embedded.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    RequestMetadata,
)
import pydantic
from typing import List, Optional, Union
from typing_extensions import Annotated, NotRequired, TypeAliasType, TypedDict


class PostCompaniesPayrollSkipCompanyUUIDPayrollType(str, Enum):
    r"""Payroll type"""

    REGULAR = "Regular"
    HIRED_EMPLOYEE = "Hired employee"
    DISMISSED_EMPLOYEE = "Dismissed employee"
    TRANSITION_FROM_OLD_PAY_SCHEDULE = "Transition from old pay schedule"


class PostCompaniesPayrollSkipCompanyUUIDRequestBodyTypedDict(TypedDict):
    payroll_type: PostCompaniesPayrollSkipCompanyUUIDPayrollType
    r"""Payroll type"""
    start_date: NotRequired[str]
    r"""Pay period start date"""
    end_date: NotRequired[str]
    r"""Pay period end date. If left empty, defaults to today's date."""
    pay_schedule_uuid: NotRequired[str]
    r"""The UUID of the pay schedule"""
    employee_uuids: NotRequired[List[str]]
    r"""An array of employees. This field is only applicable to new hire payroll and termination payroll"""


class PostCompaniesPayrollSkipCompanyUUIDRequestBody(BaseModel):
    payroll_type: PostCompaniesPayrollSkipCompanyUUIDPayrollType
    r"""Payroll type"""

    start_date: Optional[str] = None
    r"""Pay period start date"""

    end_date: Optional[str] = None
    r"""Pay period end date. If left empty, defaults to today's date."""

    pay_schedule_uuid: Optional[str] = None
    r"""The UUID of the pay schedule"""

    employee_uuids: Optional[List[str]] = None
    r"""An array of employees. This field is only applicable to new hire payroll and termination payroll"""


class PostCompaniesPayrollSkipCompanyUUIDRequestTypedDict(TypedDict):
    company_uuid: str
    r"""The UUID of the company"""
    request_body: PostCompaniesPayrollSkipCompanyUUIDRequestBodyTypedDict
    x_gusto_api_version: NotRequired[VersionHeader]
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""


class PostCompaniesPayrollSkipCompanyUUIDRequest(BaseModel):
    company_uuid: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The UUID of the company"""

    request_body: Annotated[
        PostCompaniesPayrollSkipCompanyUUIDRequestBody,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]

    x_gusto_api_version: Annotated[
        Optional[VersionHeader],
        pydantic.Field(alias="X-Gusto-API-Version"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""


PostCompaniesPayrollSkipCompanyUUIDResponseBodyUnion = TypeAliasType(
    "PostCompaniesPayrollSkipCompanyUUIDResponseBodyUnion",
    Union[UnprocessableEntityErrorObjectData, PayrollBlockersErrorData],
)
r"""Unprocessable Entity"""


class PostCompaniesPayrollSkipCompanyUUIDResponseBody(Exception):
    r"""Unprocessable Entity"""

    data: PostCompaniesPayrollSkipCompanyUUIDResponseBodyUnion

    def __init__(self, data: PostCompaniesPayrollSkipCompanyUUIDResponseBodyUnion):
        self.data = data

    def __str__(self) -> str:
        return utils.marshal_json(
            self.data, PostCompaniesPayrollSkipCompanyUUIDResponseBodyUnion
        )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gusto_embedded.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class EarningTypeTypedDict(TypedDict):
    r"""Example response"""

    uuid: str
    r"""The ID of the earning type."""
    name: NotRequired[str]
    r"""The name of the earning type."""


class EarningType(BaseModel):
    r"""Example response"""

    uuid: str
    r"""The ID of the earning type."""

    name: Optional[str] = None
    r"""The name of the earning type."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .versionheader import VersionHeader
from gusto_embedded.types import BaseModel, Nullable, UNSET_SENTINEL
from gusto_embedded.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    RequestMetadata,
)
import pydantic
from pydantic import model_serializer
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class LiabilitySelectionsTypedDict(TypedDict):
    tax_id: int
    r"""The ID of the tax."""
    last_unpaid_external_payroll_uuid: Nullable[str]
    r"""The UUID of the last unpaid external payroll uuid. It should be null when the full amount of tax liability has been paid."""
    unpaid_liability_amount: float
    r"""A selection of unpaid liability amount."""


class LiabilitySelections(BaseModel):
    tax_id: int
    r"""The ID of the tax."""

    last_unpaid_external_payroll_uuid: Nullable[str]
    r"""The UUID of the last unpaid external payroll uuid. It should be null when the full amount of tax liability has been paid."""

    unpaid_liability_amount: float
    r"""A selection of unpaid liability amount."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = []
        nullable_fields = ["last_unpaid_external_payroll_uuid"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m


class PutV1TaxLiabilitiesRequestBodyTypedDict(TypedDict):
    liability_selections: NotRequired[List[LiabilitySelectionsTypedDict]]


class PutV1TaxLiabilitiesRequestBody(BaseModel):
    liability_selections: Optional[List[LiabilitySelections]] = None


class PutV1TaxLiabilitiesRequestTypedDict(TypedDict):
    company_uuid: str
    r"""The UUID of the company"""
    request_body: PutV1TaxLiabilitiesRequestBodyTypedDict
    x_gusto_api_version: NotRequired[VersionHeader]
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""


class PutV1TaxLiabilitiesRequest(BaseModel):
    company_uuid: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The UUID of the company"""

    request_body: Annotated[
        PutV1TaxLiabilitiesRequestBody,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]

    x_gusto_api_version: Annotated[
        Optional[VersionHeader],
        pydantic.Field(alias="X-Gusto-API-Version"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .versionheader import VersionHeader
from enum import Enum
from gusto_embedded.types import BaseModel
from gusto_embedded.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    RequestMetadata,
)
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class PutV1PartnerManagedCompaniesCompanyUUIDMigrateRequestBodyTypedDict(TypedDict):
    email: str
    r"""Email of the company signatory who is authorized to accept our [Terms of Service](https://flows.gusto.com/terms) and migration decision. You can retrieve the signatory email from the `GET /v/1/companies/{company_id}/signatories` endpoint."""
    ip_address: str
    r"""The IP address of the signatory who viewed and accepted the Terms of Service."""
    external_user_id: str
    r"""The signatory's user ID on your platform."""


class PutV1PartnerManagedCompaniesCompanyUUIDMigrateRequestBody(BaseModel):
    email: str
    r"""Email of the company signatory who is authorized to accept our [Terms of Service](https://flows.gusto.com/terms) and migration decision. You can retrieve the signatory email from the `GET /v/1/companies/{company_id}/signatories` endpoint."""

    ip_address: str
    r"""The IP address of the signatory who viewed and accepted the Terms of Service."""

    external_user_id: str
    r"""The signatory's user ID on your platform."""


class PutV1PartnerManagedCompaniesCompanyUUIDMigrateRequestTypedDict(TypedDict):
    company_uuid: str
    r"""The UUID of the company"""
    request_body: PutV1PartnerManagedCompaniesCompanyUUIDMigrateRequestBodyTypedDict
    x_gusto_api_version: NotRequired[VersionHeader]
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""


class PutV1PartnerManagedCompaniesCompanyUUIDMigrateRequest(BaseModel):
    company_uuid: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The UUID of the company"""

    request_body: Annotated[
        PutV1PartnerManagedCompaniesCompanyUUIDMigrateRequestBody,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]

    x_gusto_api_version: Annotated[
        Optional[VersionHeader],
        pydantic.Field(alias="X-Gusto-API-Version"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""


class MigrationStatus(str, Enum):
    r"""The migration status. 'success' is the only valid return value."""

    SUCCESS = "success"


class PutV1PartnerManagedCompaniesCompanyUUIDMigrateResponseBodyTypedDict(TypedDict):
    r"""Example response"""

    company_uuid: NotRequired[str]
    r"""The company UUID"""
    migration_status: NotRequired[MigrationStatus]
    r"""The migration status. 'success' is the only valid return value."""


class PutV1PartnerManagedCompaniesCompanyUUIDMigrateResponseBody(BaseModel):
    r"""Example response"""

    company_uuid: Optional[str] = None
    r"""The company UUID"""

    migration_status: Optional[MigrationStatus] = None
    r"""The migration status. 'success' is the only valid return value."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gusto_embedded.types import BaseModel
from typing import List, Optional
from typing_extensions import NotRequired, TypedDict


class DepartmentEmployeesTypedDict(TypedDict):
    uuid: NotRequired[str]


class DepartmentEmployees(BaseModel):
    uuid: Optional[str] = None


class ContractorsModelTypedDict(TypedDict):
    uuid: NotRequired[str]


class ContractorsModel(BaseModel):
    uuid: Optional[str] = None


class DepartmentTypedDict(TypedDict):
    version: NotRequired[str]
    r"""The current version of the object. See the [versioning guide](https://docs.gusto.com/embedded-payroll/docs/idempotency) for information on how to use this field."""
    uuid: NotRequired[str]
    r"""The UUID of the department"""
    company_uuid: NotRequired[str]
    r"""The UUID of the company"""
    title: NotRequired[str]
    r"""Name of the department"""
    employees: NotRequired[List[DepartmentEmployeesTypedDict]]
    r"""Array of employees assigned to the department."""
    contractors: NotRequired[List[ContractorsModelTypedDict]]
    r"""Array of contractors assigned to the department."""


class Department(BaseModel):
    version: Optional[str] = None
    r"""The current version of the object. See the [versioning guide](https://docs.gusto.com/embedded-payroll/docs/idempotency) for information on how to use this field."""

    uuid: Optional[str] = None
    r"""The UUID of the department"""

    company_uuid: Optional[str] = None
    r"""The UUID of the company"""

    title: Optional[str] = None
    r"""Name of the department"""

    employees: Optional[List[DepartmentEmployees]] = None
    r"""Array of employees assigned to the department."""

    contractors: Optional[List[ContractorsModel]] = None
    r"""Array of contractors assigned to the department."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .versionheader import VersionHeader
from gusto_embedded.types import BaseModel
from gusto_embedded.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    RequestMetadata,
)
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class PostPartnerManagedCompaniesCompanyUUIDAcceptTermsOfServiceRequestBodyTypedDict(
    TypedDict
):
    email: str
    r"""The user's email address on Gusto. You can retrieve the user's email via company's `/admins`, `/employees`, `/signatories`, and `/contractors` endpoints."""
    ip_address: str
    r"""The IP address of the user who viewed and accepted the Terms of Service."""
    external_user_id: str
    r"""The user ID on your platform."""


class PostPartnerManagedCompaniesCompanyUUIDAcceptTermsOfServiceRequestBody(BaseModel):
    email: str
    r"""The user's email address on Gusto. You can retrieve the user's email via company's `/admins`, `/employees`, `/signatories`, and `/contractors` endpoints."""

    ip_address: str
    r"""The IP address of the user who viewed and accepted the Terms of Service."""

    external_user_id: str
    r"""The user ID on your platform."""


class PostPartnerManagedCompaniesCompanyUUIDAcceptTermsOfServiceRequestTypedDict(
    TypedDict
):
    company_uuid: str
    r"""The UUID of the company"""
    request_body: (
        PostPartnerManagedCompaniesCompanyUUIDAcceptTermsOfServiceRequestBodyTypedDict
    )
    x_gusto_api_version: NotRequired[VersionHeader]
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""


class PostPartnerManagedCompaniesCompanyUUIDAcceptTermsOfServiceRequest(BaseModel):
    company_uuid: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The UUID of the company"""

    request_body: Annotated[
        PostPartnerManagedCompaniesCompanyUUIDAcceptTermsOfServiceRequestBody,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]

    x_gusto_api_version: Annotated[
        Optional[VersionHeader],
        pydantic.Field(alias="X-Gusto-API-Version"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""


class PostPartnerManagedCompaniesCompanyUUIDAcceptTermsOfServiceResponseBodyTypedDict(
    TypedDict
):
    r"""Example response"""

    latest_terms_accepted: NotRequired[bool]
    r"""Whether the latest terms have been accepted by the user."""


class PostPartnerManagedCompaniesCompanyUUIDAcceptTermsOfServiceResponseBody(BaseModel):
    r"""Example response"""

    latest_terms_accepted: Optional[bool] = None
    r"""Whether the latest terms have been accepted by the user."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gusto_embedded.types import (
    BaseModel,
    Nullable,
    OptionalNullable,
    UNSET,
    UNSET_SENTINEL,
)
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class FormTypedDict(TypedDict):
    r"""Example response"""

    uuid: str
    r"""The UUID of the form"""
    name: NotRequired[str]
    r"""The type identifier of the form"""
    title: NotRequired[str]
    r"""The title of the form"""
    description: NotRequired[str]
    r"""The description of the form"""
    draft: NotRequired[bool]
    r"""If the form is in a draft state. E.g. End of year tax forms may be provided in a draft state prior to being finalized."""
    year: NotRequired[Nullable[int]]
    r"""The year of this form. For some forms, e.g. tax forms, this is the year which the form represents. A W2 for January - December 2022 would be delivered in January 2023 and have a year value of 2022. This value is nullable and will not be present on all forms."""
    quarter: NotRequired[Nullable[int]]
    r"""The quarter of this form. For some forms, e.g. tax forms, this is the calendar quarter which this form represents. An Employer's Quarterly Federal Tax Return (Form 941) for April, May, June 2022 would have a quarter value of 2 (and a year value of 2022). This value is nullable and will not be present on all forms."""
    requires_signing: NotRequired[bool]
    r"""A boolean flag that indicates whether the form needs signing or not. Note that this value will change after the form is signed."""
    document_content_type: NotRequired[Nullable[str]]
    r"""The content type of the associated document. Most forms are PDFs with a content type of `application/pdf`. Some tax file packages will be zip files (containing PDFs) with a content type of `application/zip`. This attribute will be `null` when the document has not been prepared."""


class Form(BaseModel):
    r"""Example response"""

    uuid: str
    r"""The UUID of the form"""

    name: Optional[str] = None
    r"""The type identifier of the form"""

    title: Optional[str] = None
    r"""The title of the form"""

    description: Optional[str] = None
    r"""The description of the form"""

    draft: Optional[bool] = None
    r"""If the form is in a draft state. E.g. End of year tax forms may be provided in a draft state prior to being finalized."""

    year: OptionalNullable[int] = UNSET
    r"""The year of this form. For some forms, e.g. tax forms, this is the year which the form represents. A W2 for January - December 2022 would be delivered in January 2023 and have a year value of 2022. This value is nullable and will not be present on all forms."""

    quarter: OptionalNullable[int] = UNSET
    r"""The quarter of this form. For some forms, e.g. tax forms, this is the calendar quarter which this form represents. An Employer's Quarterly Federal Tax Return (Form 941) for April, May, June 2022 would have a quarter value of 2 (and a year value of 2022). This value is nullable and will not be present on all forms."""

    requires_signing: Optional[bool] = None
    r"""A boolean flag that indicates whether the form needs signing or not. Note that this value will change after the form is signed."""

    document_content_type: OptionalNullable[str] = UNSET
    r"""The content type of the associated document. Most forms are PDFs with a content type of `application/pdf`. Some tax file packages will be zip files (containing PDFs) with a content type of `application/zip`. This attribute will be `null` when the document has not been prepared."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = [
            "name",
            "title",
            "description",
            "draft",
            "year",
            "quarter",
            "requires_signing",
            "document_content_type",
        ]
        nullable_fields = ["year", "quarter", "document_content_type"]
        null_default_fields = []

        serialized = handler(self)

        m = {}

        for n, f in self.model_fields.items():
            k = f.alias or n
            val = serialized.get(k)
            serialized.pop(k, None)

            optional_nullable = k in optional_fields and k in nullable_fields
            is_set = (
                self.__pydantic_fields_set__.intersection({n})
                or k in null_default_fields
            )  # pylint: disable=no-member

            if val is not None and val != UNSET_SENTINEL:
                m[k] = val
            elif val != UNSET_SENTINEL and (
                not k in optional_fields or (optional_nullable and is_set)
            ):
                m[k] = val

        return m



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .versionheader import VersionHeader
from gusto_embedded.types import BaseModel
from gusto_embedded.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    RequestMetadata,
)
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class PutVersionTimeOffPoliciesTimeOffPolicyUUIDAddEmployeesEmployeesTypedDict(
    TypedDict
):
    uuid: NotRequired[str]
    balance: NotRequired[str]


class PutVersionTimeOffPoliciesTimeOffPolicyUUIDAddEmployeesEmployees(BaseModel):
    uuid: Optional[str] = None

    balance: Optional[str] = None


class PutVersionTimeOffPoliciesTimeOffPolicyUUIDAddEmployeesRequestBodyTypedDict(
    TypedDict
):
    r"""A list of employee objects containing the employee uuid"""

    employees: NotRequired[
        List[PutVersionTimeOffPoliciesTimeOffPolicyUUIDAddEmployeesEmployeesTypedDict]
    ]


class PutVersionTimeOffPoliciesTimeOffPolicyUUIDAddEmployeesRequestBody(BaseModel):
    r"""A list of employee objects containing the employee uuid"""

    employees: Optional[
        List[PutVersionTimeOffPoliciesTimeOffPolicyUUIDAddEmployeesEmployees]
    ] = None


class PutVersionTimeOffPoliciesTimeOffPolicyUUIDAddEmployeesRequestTypedDict(TypedDict):
    time_off_policy_uuid: str
    r"""The UUID of the company time off policy"""
    request_body: (
        PutVersionTimeOffPoliciesTimeOffPolicyUUIDAddEmployeesRequestBodyTypedDict
    )
    r"""A list of employee objects containing the employee uuid"""
    x_gusto_api_version: NotRequired[VersionHeader]
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""


class PutVersionTimeOffPoliciesTimeOffPolicyUUIDAddEmployeesRequest(BaseModel):
    time_off_policy_uuid: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The UUID of the company time off policy"""

    request_body: Annotated[
        PutVersionTimeOffPoliciesTimeOffPolicyUUIDAddEmployeesRequestBody,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]
    r"""A list of employee objects containing the employee uuid"""

    x_gusto_api_version: Annotated[
        Optional[VersionHeader],
        pydantic.Field(alias="X-Gusto-API-Version"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gusto_embedded.types import BaseModel
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class AuthenticationTypedDict(TypedDict):
    r"""Example response"""

    access_token: NotRequired[str]
    r"""A new access token that can be used for subsequent authenticated requests"""
    token_type: NotRequired[str]
    r"""The literal string 'bearer'"""
    expires_in: NotRequired[float]
    r"""The TTL of this token. After this amount of time, you must hit the refresh token endpoint to continue making authenticated requests."""
    refresh_token: NotRequired[str]
    r"""A token that must be passed to the refresh token endpoint to get a new authenticated token."""
    created_at: NotRequired[str]
    r"""Datetime for when the new access token is created."""
    scope: NotRequired[str]
    r"""All of the scopes for which the access token provides access."""


class Authentication(BaseModel):
    r"""Example response"""

    access_token: Optional[str] = None
    r"""A new access token that can be used for subsequent authenticated requests"""

    token_type: Optional[str] = "bearer"
    r"""The literal string 'bearer'"""

    expires_in: Optional[float] = 7200
    r"""The TTL of this token. After this amount of time, you must hit the refresh token endpoint to continue making authenticated requests."""

    refresh_token: Optional[str] = None
    r"""A token that must be passed to the refresh token endpoint to get a new authenticated token."""

    created_at: Optional[str] = None
    r"""Datetime for when the new access token is created."""

    scope: Optional[str] = None
    r"""All of the scopes for which the access token provides access."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .versionheader import VersionHeader
from gusto_embedded.types import BaseModel
from gusto_embedded.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    RequestMetadata,
)
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class PutV1ContractorDocumentSignFieldsTypedDict(TypedDict):
    key: NotRequired[str]
    r"""Unique identifier of the field"""
    value: NotRequired[str]
    r"""Value for the field"""


class PutV1ContractorDocumentSignFields(BaseModel):
    key: Optional[str] = None
    r"""Unique identifier of the field"""

    value: Optional[str] = None
    r"""Value for the field"""


class PutV1ContractorDocumentSignRequestBodyTypedDict(TypedDict):
    fields: List[PutV1ContractorDocumentSignFieldsTypedDict]
    r"""List of fields and the values they will be set to."""
    agree: bool
    r"""Whether you agree to sign electronically"""
    signed_by_ip_address: NotRequired[str]
    r"""The IP address of the signatory who signed the form. You must provide the IP address with either this parameter OR you can leave out this parameter and set the IP address in the request header using the `x-gusto-client-ip` header instead."""


class PutV1ContractorDocumentSignRequestBody(BaseModel):
    fields: List[PutV1ContractorDocumentSignFields]
    r"""List of fields and the values they will be set to."""

    agree: bool
    r"""Whether you agree to sign electronically"""

    signed_by_ip_address: Optional[str] = None
    r"""The IP address of the signatory who signed the form. You must provide the IP address with either this parameter OR you can leave out this parameter and set the IP address in the request header using the `x-gusto-client-ip` header instead."""


class PutV1ContractorDocumentSignRequestTypedDict(TypedDict):
    document_uuid: str
    r"""The ID or UUID of the document"""
    request_body: PutV1ContractorDocumentSignRequestBodyTypedDict
    x_gusto_client_ip: NotRequired[str]
    r"""Optional header to supply the IP address. This can be used to supply the IP address for signature endpoints instead of the signed_by_ip_address parameter."""
    x_gusto_api_version: NotRequired[VersionHeader]
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""


class PutV1ContractorDocumentSignRequest(BaseModel):
    document_uuid: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The ID or UUID of the document"""

    request_body: Annotated[
        PutV1ContractorDocumentSignRequestBody,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]

    x_gusto_client_ip: Annotated[
        Optional[str],
        pydantic.Field(alias="x-gusto-client-ip"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = None
    r"""Optional header to supply the IP address. This can be used to supply the IP address for signature endpoints instead of the signed_by_ip_address parameter."""

    x_gusto_api_version: Annotated[
        Optional[VersionHeader],
        pydantic.Field(alias="X-Gusto-API-Version"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from gusto_embedded import models, utils
from gusto_embedded._hooks import HookContext
from gusto_embedded.types import OptionalNullable, UNSET
from gusto_embedded.utils import get_security_from_env
from typing import Any, List, Mapping, Optional


class EmployeeForms(BaseSDK):
    def generate_w2(
        self,
        *,
        employee_id: str,
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01,
        year: Optional[int] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.PostV1SandboxGenerateW2Form:
        r"""Generate a W2 form [DEMO]

        > 🚧 Demo action
        >
        > This action is only available in the Demo environment

        Generates a W2 document for testing purposes.

        scope: `employees:write`

        :param employee_id: The employee UUID.
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param year: Must be equal to or more recent than 2015. If not specified, defaults to the previous year.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.PostV1SandboxGenerateW2Request(
            x_gusto_api_version=x_gusto_api_version,
            request_body=models.PostV1SandboxGenerateW2RequestBody(
                employee_id=employee_id,
                year=year,
            ),
        )

        req = self._build_request(
            method="POST",
            path="/v1/sandbox/generate_w2",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.request_body,
                False,
                False,
                "json",
                models.PostV1SandboxGenerateW2RequestBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="post-v1-sandbox-generate_w2",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(
                http_res.text, models.PostV1SandboxGenerateW2Form
            )
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableEntityErrorObjectData
            )
            raise models.UnprocessableEntityErrorObject(data=response_data)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def generate_w2_async(
        self,
        *,
        employee_id: str,
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01,
        year: Optional[int] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.PostV1SandboxGenerateW2Form:
        r"""Generate a W2 form [DEMO]

        > 🚧 Demo action
        >
        > This action is only available in the Demo environment

        Generates a W2 document for testing purposes.

        scope: `employees:write`

        :param employee_id: The employee UUID.
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param year: Must be equal to or more recent than 2015. If not specified, defaults to the previous year.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.PostV1SandboxGenerateW2Request(
            x_gusto_api_version=x_gusto_api_version,
            request_body=models.PostV1SandboxGenerateW2RequestBody(
                employee_id=employee_id,
                year=year,
            ),
        )

        req = self._build_request_async(
            method="POST",
            path="/v1/sandbox/generate_w2",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=False,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.request_body,
                False,
                False,
                "json",
                models.PostV1SandboxGenerateW2RequestBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="post-v1-sandbox-generate_w2",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(
                http_res.text, models.PostV1SandboxGenerateW2Form
            )
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableEntityErrorObjectData
            )
            raise models.UnprocessableEntityErrorObject(data=response_data)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def list(
        self,
        *,
        employee_id: str,
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> List[models.Form]:
        r"""Get all employee forms

        Get a list of all employee's forms

        scope: `employee_forms:read`

        :param employee_id: The UUID of the employee
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetV1EmployeeFormsRequest(
            employee_id=employee_id,
            x_gusto_api_version=x_gusto_api_version,
        )

        req = self._build_request(
            method="GET",
            path="/v1/employees/{employee_id}/forms",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="get-v1-employee-forms",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, List[models.Form])
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def list_async(
        self,
        *,
        employee_id: str,
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> List[models.Form]:
        r"""Get all employee forms

        Get a list of all employee's forms

        scope: `employee_forms:read`

        :param employee_id: The UUID of the employee
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetV1EmployeeFormsRequest(
            employee_id=employee_id,
            x_gusto_api_version=x_gusto_api_version,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/employees/{employee_id}/forms",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="get-v1-employee-forms",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, List[models.Form])
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def get(
        self,
        *,
        employee_id: str,
        form_id: str,
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.Form:
        r"""Get an employee form

        Get an employee form

        scope: `employee_forms:read`

        :param employee_id: The UUID of the employee
        :param form_id: The UUID of the form
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetV1EmployeeFormRequest(
            employee_id=employee_id,
            form_id=form_id,
            x_gusto_api_version=x_gusto_api_version,
        )

        req = self._build_request(
            method="GET",
            path="/v1/employees/{employee_id}/forms/{form_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="get-v1-employee-form",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.Form)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def get_async(
        self,
        *,
        employee_id: str,
        form_id: str,
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.Form:
        r"""Get an employee form

        Get an employee form

        scope: `employee_forms:read`

        :param employee_id: The UUID of the employee
        :param form_id: The UUID of the form
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetV1EmployeeFormRequest(
            employee_id=employee_id,
            form_id=form_id,
            x_gusto_api_version=x_gusto_api_version,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/employees/{employee_id}/forms/{form_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="get-v1-employee-form",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.Form)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def get_pdf(
        self,
        *,
        employee_id: str,
        form_id: str,
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.FormPdf:
        r"""Get the employee form pdf

        Get the link to the employee form PDF

        scope: `employee_forms:read`

        :param employee_id: The UUID of the employee
        :param form_id: The UUID of the form
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetV1EmployeeFormPdfRequest(
            employee_id=employee_id,
            form_id=form_id,
            x_gusto_api_version=x_gusto_api_version,
        )

        req = self._build_request(
            method="GET",
            path="/v1/employees/{employee_id}/forms/{form_id}/pdf",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="get-v1-employee-form-pdf",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.FormPdf)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def get_pdf_async(
        self,
        *,
        employee_id: str,
        form_id: str,
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.FormPdf:
        r"""Get the employee form pdf

        Get the link to the employee form PDF

        scope: `employee_forms:read`

        :param employee_id: The UUID of the employee
        :param form_id: The UUID of the form
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.GetV1EmployeeFormPdfRequest(
            employee_id=employee_id,
            form_id=form_id,
            x_gusto_api_version=x_gusto_api_version,
        )

        req = self._build_request_async(
            method="GET",
            path="/v1/employees/{employee_id}/forms/{form_id}/pdf",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="get-v1-employee-form-pdf",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "4XX", "5XX"],
            retry_config=retry_config,
        )

        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.FormPdf)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    def sign(
        self,
        *,
        employee_id: str,
        form_id: str,
        signature_text: str,
        agree: bool,
        x_gusto_client_ip: Optional[str] = None,
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01,
        signed_by_ip_address: Optional[str] = None,
        preparer: Optional[bool] = None,
        preparer_first_name: Optional[str] = None,
        preparer_last_name: Optional[str] = None,
        preparer_street_1: Optional[str] = None,
        preparer_street_2: Optional[str] = None,
        preparer_city: Optional[str] = None,
        preparer_state: Optional[str] = None,
        preparer_zip: Optional[str] = None,
        preparer_agree: Optional[str] = None,
        preparer2: Optional[bool] = None,
        preparer2_first_name: Optional[str] = None,
        preparer2_last_name: Optional[str] = None,
        preparer2_street_1: Optional[str] = None,
        preparer2_street_2: Optional[str] = None,
        preparer2_city: Optional[str] = None,
        preparer2_state: Optional[str] = None,
        preparer2_zip: Optional[str] = None,
        preparer2_agree: Optional[str] = None,
        preparer3: Optional[bool] = None,
        preparer3_first_name: Optional[str] = None,
        preparer3_last_name: Optional[str] = None,
        preparer3_street_1: Optional[str] = None,
        preparer3_street_2: Optional[str] = None,
        preparer3_city: Optional[str] = None,
        preparer3_state: Optional[str] = None,
        preparer3_zip: Optional[str] = None,
        preparer3_agree: Optional[str] = None,
        preparer4: Optional[bool] = None,
        preparer4_first_name: Optional[str] = None,
        preparer4_last_name: Optional[str] = None,
        preparer4_street_1: Optional[str] = None,
        preparer4_street_2: Optional[str] = None,
        preparer4_city: Optional[str] = None,
        preparer4_state: Optional[str] = None,
        preparer4_zip: Optional[str] = None,
        preparer4_agree: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.Form:
        r"""Sign an employee form

        Sign an employee form.

        The optional preparer attributes are only valid for I-9 form. When a preparer is used, the
        first name, last name, street address, city, state, and zip for that preparer are all required.

        scope: `employee_forms:sign`

        :param employee_id: The UUID of the employee
        :param form_id: The UUID of the form
        :param signature_text: The signature
        :param agree: Whether you agree to sign electronically
        :param x_gusto_client_ip: Optional header to supply the IP address. This can be used to supply the IP address for signature endpoints instead of the signed_by_ip_address parameter.
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param signed_by_ip_address: The IP address of the signatory who signed the form. Both IPv4 AND IPv6 are supported. You must provide the IP address with either this parameter OR you can leave out this parameter and set the IP address in the request header using the `x-gusto-client-ip` header instead.
        :param preparer: Whether there is a preparer
        :param preparer_first_name:
        :param preparer_last_name:
        :param preparer_street_1:
        :param preparer_street_2:
        :param preparer_city:
        :param preparer_state:
        :param preparer_zip:
        :param preparer_agree: Whether preparer agrees to sign electronically
        :param preparer2: Whether there is a 2nd preparer
        :param preparer2_first_name:
        :param preparer2_last_name:
        :param preparer2_street_1:
        :param preparer2_street_2:
        :param preparer2_city:
        :param preparer2_state:
        :param preparer2_zip:
        :param preparer2_agree: Whether 2nd preparer agrees to sign electronically
        :param preparer3: Whether there is a 3rd preparer
        :param preparer3_first_name:
        :param preparer3_last_name:
        :param preparer3_street_1:
        :param preparer3_street_2:
        :param preparer3_city:
        :param preparer3_state:
        :param preparer3_zip:
        :param preparer3_agree: Whether 3rd preparer agrees to sign electronically
        :param preparer4: Whether there is a 4th preparer
        :param preparer4_first_name:
        :param preparer4_last_name:
        :param preparer4_street_1:
        :param preparer4_street_2:
        :param preparer4_city:
        :param preparer4_state:
        :param preparer4_zip:
        :param preparer4_agree: Whether 4th preparer agrees to sign electronically
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.PutV1EmployeeFormSignRequest(
            employee_id=employee_id,
            form_id=form_id,
            x_gusto_client_ip=x_gusto_client_ip,
            x_gusto_api_version=x_gusto_api_version,
            request_body=models.PutV1EmployeeFormSignRequestBody(
                signature_text=signature_text,
                agree=agree,
                signed_by_ip_address=signed_by_ip_address,
                preparer=preparer,
                preparer_first_name=preparer_first_name,
                preparer_last_name=preparer_last_name,
                preparer_street_1=preparer_street_1,
                preparer_street_2=preparer_street_2,
                preparer_city=preparer_city,
                preparer_state=preparer_state,
                preparer_zip=preparer_zip,
                preparer_agree=preparer_agree,
                preparer2=preparer2,
                preparer2_first_name=preparer2_first_name,
                preparer2_last_name=preparer2_last_name,
                preparer2_street_1=preparer2_street_1,
                preparer2_street_2=preparer2_street_2,
                preparer2_city=preparer2_city,
                preparer2_state=preparer2_state,
                preparer2_zip=preparer2_zip,
                preparer2_agree=preparer2_agree,
                preparer3=preparer3,
                preparer3_first_name=preparer3_first_name,
                preparer3_last_name=preparer3_last_name,
                preparer3_street_1=preparer3_street_1,
                preparer3_street_2=preparer3_street_2,
                preparer3_city=preparer3_city,
                preparer3_state=preparer3_state,
                preparer3_zip=preparer3_zip,
                preparer3_agree=preparer3_agree,
                preparer4=preparer4,
                preparer4_first_name=preparer4_first_name,
                preparer4_last_name=preparer4_last_name,
                preparer4_street_1=preparer4_street_1,
                preparer4_street_2=preparer4_street_2,
                preparer4_city=preparer4_city,
                preparer4_state=preparer4_state,
                preparer4_zip=preparer4_zip,
                preparer4_agree=preparer4_agree,
            ),
        )

        req = self._build_request(
            method="PUT",
            path="/v1/employees/{employee_id}/forms/{form_id}/sign",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.request_body,
                False,
                False,
                "json",
                models.PutV1EmployeeFormSignRequestBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="put-v1-employee-form-sign",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.Form)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableEntityErrorObjectData
            )
            raise models.UnprocessableEntityErrorObject(data=response_data)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = utils.stream_to_text(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = utils.stream_to_text(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )

    async def sign_async(
        self,
        *,
        employee_id: str,
        form_id: str,
        signature_text: str,
        agree: bool,
        x_gusto_client_ip: Optional[str] = None,
        x_gusto_api_version: Optional[
            models.VersionHeader
        ] = models.VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01,
        signed_by_ip_address: Optional[str] = None,
        preparer: Optional[bool] = None,
        preparer_first_name: Optional[str] = None,
        preparer_last_name: Optional[str] = None,
        preparer_street_1: Optional[str] = None,
        preparer_street_2: Optional[str] = None,
        preparer_city: Optional[str] = None,
        preparer_state: Optional[str] = None,
        preparer_zip: Optional[str] = None,
        preparer_agree: Optional[str] = None,
        preparer2: Optional[bool] = None,
        preparer2_first_name: Optional[str] = None,
        preparer2_last_name: Optional[str] = None,
        preparer2_street_1: Optional[str] = None,
        preparer2_street_2: Optional[str] = None,
        preparer2_city: Optional[str] = None,
        preparer2_state: Optional[str] = None,
        preparer2_zip: Optional[str] = None,
        preparer2_agree: Optional[str] = None,
        preparer3: Optional[bool] = None,
        preparer3_first_name: Optional[str] = None,
        preparer3_last_name: Optional[str] = None,
        preparer3_street_1: Optional[str] = None,
        preparer3_street_2: Optional[str] = None,
        preparer3_city: Optional[str] = None,
        preparer3_state: Optional[str] = None,
        preparer3_zip: Optional[str] = None,
        preparer3_agree: Optional[str] = None,
        preparer4: Optional[bool] = None,
        preparer4_first_name: Optional[str] = None,
        preparer4_last_name: Optional[str] = None,
        preparer4_street_1: Optional[str] = None,
        preparer4_street_2: Optional[str] = None,
        preparer4_city: Optional[str] = None,
        preparer4_state: Optional[str] = None,
        preparer4_zip: Optional[str] = None,
        preparer4_agree: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
        http_headers: Optional[Mapping[str, str]] = None,
    ) -> models.Form:
        r"""Sign an employee form

        Sign an employee form.

        The optional preparer attributes are only valid for I-9 form. When a preparer is used, the
        first name, last name, street address, city, state, and zip for that preparer are all required.

        scope: `employee_forms:sign`

        :param employee_id: The UUID of the employee
        :param form_id: The UUID of the form
        :param signature_text: The signature
        :param agree: Whether you agree to sign electronically
        :param x_gusto_client_ip: Optional header to supply the IP address. This can be used to supply the IP address for signature endpoints instead of the signed_by_ip_address parameter.
        :param x_gusto_api_version: Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used.
        :param signed_by_ip_address: The IP address of the signatory who signed the form. Both IPv4 AND IPv6 are supported. You must provide the IP address with either this parameter OR you can leave out this parameter and set the IP address in the request header using the `x-gusto-client-ip` header instead.
        :param preparer: Whether there is a preparer
        :param preparer_first_name:
        :param preparer_last_name:
        :param preparer_street_1:
        :param preparer_street_2:
        :param preparer_city:
        :param preparer_state:
        :param preparer_zip:
        :param preparer_agree: Whether preparer agrees to sign electronically
        :param preparer2: Whether there is a 2nd preparer
        :param preparer2_first_name:
        :param preparer2_last_name:
        :param preparer2_street_1:
        :param preparer2_street_2:
        :param preparer2_city:
        :param preparer2_state:
        :param preparer2_zip:
        :param preparer2_agree: Whether 2nd preparer agrees to sign electronically
        :param preparer3: Whether there is a 3rd preparer
        :param preparer3_first_name:
        :param preparer3_last_name:
        :param preparer3_street_1:
        :param preparer3_street_2:
        :param preparer3_city:
        :param preparer3_state:
        :param preparer3_zip:
        :param preparer3_agree: Whether 3rd preparer agrees to sign electronically
        :param preparer4: Whether there is a 4th preparer
        :param preparer4_first_name:
        :param preparer4_last_name:
        :param preparer4_street_1:
        :param preparer4_street_2:
        :param preparer4_city:
        :param preparer4_state:
        :param preparer4_zip:
        :param preparer4_agree: Whether 4th preparer agrees to sign electronically
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        :param http_headers: Additional headers to set or replace on requests.
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url
        else:
            base_url = self._get_url(base_url, url_variables)

        request = models.PutV1EmployeeFormSignRequest(
            employee_id=employee_id,
            form_id=form_id,
            x_gusto_client_ip=x_gusto_client_ip,
            x_gusto_api_version=x_gusto_api_version,
            request_body=models.PutV1EmployeeFormSignRequestBody(
                signature_text=signature_text,
                agree=agree,
                signed_by_ip_address=signed_by_ip_address,
                preparer=preparer,
                preparer_first_name=preparer_first_name,
                preparer_last_name=preparer_last_name,
                preparer_street_1=preparer_street_1,
                preparer_street_2=preparer_street_2,
                preparer_city=preparer_city,
                preparer_state=preparer_state,
                preparer_zip=preparer_zip,
                preparer_agree=preparer_agree,
                preparer2=preparer2,
                preparer2_first_name=preparer2_first_name,
                preparer2_last_name=preparer2_last_name,
                preparer2_street_1=preparer2_street_1,
                preparer2_street_2=preparer2_street_2,
                preparer2_city=preparer2_city,
                preparer2_state=preparer2_state,
                preparer2_zip=preparer2_zip,
                preparer2_agree=preparer2_agree,
                preparer3=preparer3,
                preparer3_first_name=preparer3_first_name,
                preparer3_last_name=preparer3_last_name,
                preparer3_street_1=preparer3_street_1,
                preparer3_street_2=preparer3_street_2,
                preparer3_city=preparer3_city,
                preparer3_state=preparer3_state,
                preparer3_zip=preparer3_zip,
                preparer3_agree=preparer3_agree,
                preparer4=preparer4,
                preparer4_first_name=preparer4_first_name,
                preparer4_last_name=preparer4_last_name,
                preparer4_street_1=preparer4_street_1,
                preparer4_street_2=preparer4_street_2,
                preparer4_city=preparer4_city,
                preparer4_state=preparer4_state,
                preparer4_zip=preparer4_zip,
                preparer4_agree=preparer4_agree,
            ),
        )

        req = self._build_request_async(
            method="PUT",
            path="/v1/employees/{employee_id}/forms/{form_id}/sign",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            http_headers=http_headers,
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.request_body,
                False,
                False,
                "json",
                models.PutV1EmployeeFormSignRequestBody,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                base_url=base_url or "",
                operation_id="put-v1-employee-form-sign",
                oauth2_scopes=[],
                security_source=get_security_from_env(
                    self.sdk_configuration.security, models.Security
                ),
            ),
            request=req,
            error_status_codes=["404", "422", "4XX", "5XX"],
            retry_config=retry_config,
        )

        response_data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return utils.unmarshal_json(http_res.text, models.Form)
        if utils.match_response(http_res, "422", "application/json"):
            response_data = utils.unmarshal_json(
                http_res.text, models.UnprocessableEntityErrorObjectData
            )
            raise models.UnprocessableEntityErrorObject(data=response_data)
        if utils.match_response(http_res, ["404", "4XX"], "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )
        if utils.match_response(http_res, "5XX", "*"):
            http_res_text = await utils.stream_to_text_async(http_res)
            raise models.APIError(
                "API error occurred", http_res.status_code, http_res_text, http_res
            )

        content_type = http_res.headers.get("Content-Type")
        http_res_text = await utils.stream_to_text_async(http_res)
        raise models.APIError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res_text,
            http_res,
        )



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from enum import Enum


class CustomFieldType(str, Enum):
    r"""Input type for the custom field."""

    TEXT = "text"
    CURRENCY = "currency"
    NUMBER = "number"
    DATE = "date"
    RADIO = "radio"



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .versionheader import VersionHeader
from gusto_embedded.types import BaseModel
from gusto_embedded.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    QueryParamMetadata,
    SecurityMetadata,
)
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class GetInvoicesInvoicePeriodSecurityTypedDict(TypedDict):
    system_access_auth: str


class GetInvoicesInvoicePeriodSecurity(BaseModel):
    system_access_auth: Annotated[
        str,
        FieldMetadata(
            security=SecurityMetadata(
                scheme=True,
                scheme_type="http",
                sub_type="bearer",
                field_name="Authorization",
            )
        ),
    ]


class GetInvoicesInvoicePeriodRequestTypedDict(TypedDict):
    invoice_period: str
    r"""The month we are calculating the invoice for. Must be in YYYY-MM format"""
    page: NotRequired[int]
    r"""The page that is requested. When unspecified, will load all objects unless endpoint forces pagination."""
    per: NotRequired[int]
    r"""Number of objects per page. For majority of endpoints will default to 25"""
    company_uuids: NotRequired[str]
    r"""Filter companies returned in the active_companies response, will return an error if company not active during provided invoice period. i.e. `?company_uuids=781922d8-e780-4b6b-bf74-ee303166d022,bbbca930-7322-491c-ba7f-98707a52a9c5`"""
    x_gusto_api_version: NotRequired[VersionHeader]
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""


class GetInvoicesInvoicePeriodRequest(BaseModel):
    invoice_period: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The month we are calculating the invoice for. Must be in YYYY-MM format"""

    page: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""The page that is requested. When unspecified, will load all objects unless endpoint forces pagination."""

    per: Annotated[
        Optional[int],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Number of objects per page. For majority of endpoints will default to 25"""

    company_uuids: Annotated[
        Optional[str],
        FieldMetadata(query=QueryParamMetadata(style="form", explode=True)),
    ] = None
    r"""Filter companies returned in the active_companies response, will return an error if company not active during provided invoice period. i.e. `?company_uuids=781922d8-e780-4b6b-bf74-ee303166d022,bbbca930-7322-491c-ba7f-98707a52a9c5`"""

    x_gusto_api_version: Annotated[
        Optional[VersionHeader],
        pydantic.Field(alias="X-Gusto-API-Version"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from gusto_embedded.types import BaseModel
import pydantic
from typing import List, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class NewYearsDayTypedDict(TypedDict):
    selected: NotRequired[bool]
    name: NotRequired[str]
    date_: NotRequired[str]


class NewYearsDay(BaseModel):
    selected: Optional[bool] = None

    name: Optional[str] = None

    date_: Annotated[Optional[str], pydantic.Field(alias="date")] = None


class MlkDayTypedDict(TypedDict):
    selected: NotRequired[bool]
    name: NotRequired[str]
    date_: NotRequired[str]


class MlkDay(BaseModel):
    selected: Optional[bool] = None

    name: Optional[str] = None

    date_: Annotated[Optional[str], pydantic.Field(alias="date")] = None


class PresidentsDayTypedDict(TypedDict):
    selected: NotRequired[bool]
    name: NotRequired[str]
    date_: NotRequired[str]


class PresidentsDay(BaseModel):
    selected: Optional[bool] = None

    name: Optional[str] = None

    date_: Annotated[Optional[str], pydantic.Field(alias="date")] = None


class MemorialDayTypedDict(TypedDict):
    selected: NotRequired[bool]
    name: NotRequired[str]
    date_: NotRequired[str]


class MemorialDay(BaseModel):
    selected: Optional[bool] = None

    name: Optional[str] = None

    date_: Annotated[Optional[str], pydantic.Field(alias="date")] = None


class JuneteenthTypedDict(TypedDict):
    selected: NotRequired[bool]
    name: NotRequired[str]
    date_: NotRequired[str]


class Juneteenth(BaseModel):
    selected: Optional[bool] = None

    name: Optional[str] = None

    date_: Annotated[Optional[str], pydantic.Field(alias="date")] = None


class IndependenceDayTypedDict(TypedDict):
    selected: NotRequired[bool]
    name: NotRequired[str]
    date_: NotRequired[str]


class IndependenceDay(BaseModel):
    selected: Optional[bool] = None

    name: Optional[str] = None

    date_: Annotated[Optional[str], pydantic.Field(alias="date")] = None


class LaborDayTypedDict(TypedDict):
    selected: NotRequired[bool]
    name: NotRequired[str]
    date_: NotRequired[str]


class LaborDay(BaseModel):
    selected: Optional[bool] = None

    name: Optional[str] = None

    date_: Annotated[Optional[str], pydantic.Field(alias="date")] = None


class ColumbusDayTypedDict(TypedDict):
    selected: NotRequired[bool]
    name: NotRequired[str]
    date_: NotRequired[str]


class ColumbusDay(BaseModel):
    selected: Optional[bool] = None

    name: Optional[str] = None

    date_: Annotated[Optional[str], pydantic.Field(alias="date")] = None


class VeteransDayTypedDict(TypedDict):
    selected: NotRequired[bool]
    name: NotRequired[str]
    date_: NotRequired[str]


class VeteransDay(BaseModel):
    selected: Optional[bool] = None

    name: Optional[str] = None

    date_: Annotated[Optional[str], pydantic.Field(alias="date")] = None


class ThanksgivingTypedDict(TypedDict):
    selected: NotRequired[bool]
    name: NotRequired[str]
    date_: NotRequired[str]


class Thanksgiving(BaseModel):
    selected: Optional[bool] = None

    name: Optional[str] = None

    date_: Annotated[Optional[str], pydantic.Field(alias="date")] = None


class FederalHolidaysTypedDict(TypedDict):
    new_years_day: NotRequired[NewYearsDayTypedDict]
    mlk_day: NotRequired[MlkDayTypedDict]
    presidents_day: NotRequired[PresidentsDayTypedDict]
    memorial_day: NotRequired[MemorialDayTypedDict]
    juneteenth: NotRequired[JuneteenthTypedDict]
    independence_day: NotRequired[IndependenceDayTypedDict]
    labor_day: NotRequired[LaborDayTypedDict]
    columbus_day: NotRequired[ColumbusDayTypedDict]
    veterans_day: NotRequired[VeteransDayTypedDict]
    thanksgiving: NotRequired[ThanksgivingTypedDict]


class FederalHolidays(BaseModel):
    new_years_day: Optional[NewYearsDay] = None

    mlk_day: Optional[MlkDay] = None

    presidents_day: Optional[PresidentsDay] = None

    memorial_day: Optional[MemorialDay] = None

    juneteenth: Optional[Juneteenth] = None

    independence_day: Optional[IndependenceDay] = None

    labor_day: Optional[LaborDay] = None

    columbus_day: Optional[ColumbusDay] = None

    veterans_day: Optional[VeteransDay] = None

    thanksgiving: Optional[Thanksgiving] = None


class HolidayPayPolicyEmployeesTypedDict(TypedDict):
    uuid: NotRequired[str]


class HolidayPayPolicyEmployees(BaseModel):
    uuid: Optional[str] = None


class HolidayPayPolicyTypedDict(TypedDict):
    r"""Representation of a Holiday Pay Policy"""

    version: str
    r"""The current version of the object. See the [versioning guide](https://docs.gusto.com/embedded-payroll/docs/versioning#object-layer) for information on how to use this field."""
    company_uuid: str
    r"""A unique identifier for the company owning the holiday pay policy"""
    federal_holidays: List[FederalHolidaysTypedDict]
    r"""List of the eleven supported federal holidays and their details"""
    employees: List[HolidayPayPolicyEmployeesTypedDict]
    r"""List of employee uuids under a time off policy"""


class HolidayPayPolicy(BaseModel):
    r"""Representation of a Holiday Pay Policy"""

    version: str
    r"""The current version of the object. See the [versioning guide](https://docs.gusto.com/embedded-payroll/docs/versioning#object-layer) for information on how to use this field."""

    company_uuid: str
    r"""A unique identifier for the company owning the holiday pay policy"""

    federal_holidays: List[FederalHolidays]
    r"""List of the eleven supported federal holidays and their details"""

    employees: List[HolidayPayPolicyEmployees]
    r"""List of employee uuids under a time off policy"""



"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .versionheader import VersionHeader
from gusto_embedded.types import BaseModel
from gusto_embedded.utils import (
    FieldMetadata,
    HeaderMetadata,
    PathParamMetadata,
    RequestMetadata,
)
import pydantic
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class PutV1ContractorsContractorUUIDAddressRequestBodyTypedDict(TypedDict):
    version: str
    r"""The current version of the object. See the [versioning guide](https://docs.gusto.com/embedded-payroll/docs/idempotency) for information on how to use this field."""
    street_1: NotRequired[str]
    street_2: NotRequired[str]
    city: NotRequired[str]
    state: NotRequired[str]
    zip_code: NotRequired[str]


class PutV1ContractorsContractorUUIDAddressRequestBody(BaseModel):
    version: str
    r"""The current version of the object. See the [versioning guide](https://docs.gusto.com/embedded-payroll/docs/idempotency) for information on how to use this field."""

    street_1: Optional[str] = None

    street_2: Optional[str] = None

    city: Optional[str] = None

    state: Optional[str] = None

    zip_code: Annotated[Optional[str], pydantic.Field(alias="zip")] = None


class PutV1ContractorsContractorUUIDAddressRequestTypedDict(TypedDict):
    contractor_uuid: str
    r"""The UUID of the contractor"""
    request_body: PutV1ContractorsContractorUUIDAddressRequestBodyTypedDict
    x_gusto_api_version: NotRequired[VersionHeader]
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""


class PutV1ContractorsContractorUUIDAddressRequest(BaseModel):
    contractor_uuid: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The UUID of the contractor"""

    request_body: Annotated[
        PutV1ContractorsContractorUUIDAddressRequestBody,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]

    x_gusto_api_version: Annotated[
        Optional[VersionHeader],
        pydantic.Field(alias="X-Gusto-API-Version"),
        FieldMetadata(header=HeaderMetadata(style="simple", explode=False)),
    ] = VersionHeader.TWO_THOUSAND_AND_TWENTY_FOUR_04_01
    r"""Determines the date-based API version associated with your API call. If none is provided, your application's [minimum API version](https://docs.gusto.com/embedded-payroll/docs/api-versioning#minimum-api-version) is used."""
